#include "configure.h"
#include "ui_filesystemwidget.h"

Configure::Configure() : FileSystemWidget(QIcon(":/icons/32x32/config.png"))
{
	setWindowFlags( Qt::CustomizeWindowHint | Qt::WindowCloseButtonHint );
	setWindowTitle(tr("Liberus Configuration"));
	setStyleSheet(QString("QWidget {background-color: %1};").arg(CONFIGURATIONBACKGROUNDCOLOR));
	placesModel = new PlacesModel;
	ui->placesView->setModel( placesModel );
	ui->placesView->setIconSize( QSize(CONFIGURATION_ICON_SIZE, CONFIGURATION_ICON_SIZE) );
	ui->placesView->setMinimumWidth( 300 );

	removeButtons();
	ui->lineFileInfo->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::MinimumExpanding);

	container = new QStackedWidget;
	container->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);

	ui->verticalLayout->insertWidget(0, container);
	ui->verticalLayout->setAlignment( container, Qt::AlignTop | Qt::AlignLeft);

	setGeometry( 0, 0, WinInfo::desktopSize().width(), WinInfo::desktopSize().height() - 100 );
}

void Configure::removeButtons()
{
	delete ui->copyButton;
	delete ui->pasteButton;
	delete ui->newFolderButton;
	delete ui->cutButton;
	delete ui->renameButton;
	delete ui->removeButton;
	delete ui->listViewButton;
	delete ui->iconViewButton;
	delete ui->listView;
	delete ui->iconView;
	ui->lineFileInfo->hide();
	ui->fileInfo->hide();
}

Configure::~Configure()
{
	//delete mimeConfig;
	delete userConfig;
	delete fontsConfig;
	delete languageConfig;
#ifdef Q_OS_LINUX
	if ( touchConfig != nullptr )
	{
		delete touchConfig;
	}
#endif
	delete cloudConfig;
	delete displayConfig;
    //delete shareDesktopConfig;
}

void Configure::addApplications()
{
	/*mimeConfig = new MimeConfig(0);
	addConfigurationApplication(mimeConfig);
	mimeConfig->hide();*/

	userConfig = new UserConfig(nullptr);
	addConfigurationApplication(userConfig);

	fontsConfig = new FontsConfig(nullptr);
	addConfigurationApplication(fontsConfig);

	languageConfig = new LanguageConfig(nullptr);
	addConfigurationApplication(languageConfig);

#ifdef Q_OS_LINUX
	if ( TouchConfig::isAvailable() )
	{
		touchConfig = new TouchConfig(0);
		addConfigurationApplication(touchConfig);
	} else {
		touchConfig = nullptr;
	}
#endif

	cloudConfig = new CloudConfig(nullptr);
	addConfigurationApplication(cloudConfig);

	/*shareDesktopConfig = new ShareDesktopConfig(this);
	addConfigurationApplication(shareDesktopConfig);
	shareDesktopConfig->hide();*/

	displayConfig = new DisplayConfig(nullptr);
	addConfigurationApplication(displayConfig);

	placesDoubleClicked(placesModel->index(0,0));
}

void Configure::addConfigurationApplication(ConfigurationDialog * configAppName)
{
	placesModel->appendRow(new QStandardItem(configAppName->icon(), configAppName->name()));
	container->addWidget( configAppName->widget() );
	container->layout()->setAlignment( configAppName->widget(), Qt::AlignTop );
	connect( ui->placesView, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(placesDoubleClicked(QModelIndex)));
	connect( ui->placesView, SIGNAL(clicked(QModelIndex)), this, SLOT(placesDoubleClicked(QModelIndex)));
	configAppName->widget()->setStyleSheet( COMBOSTYLE );
}

void Configure::placesDoubleClicked(QModelIndex index)
{
	container->setCurrentIndex(index.row());
	ui->addressLineEdit->setText(tr("Configuration > ") + placesModel->item( index.row() )->text() );
}

void Configure::closeEvent(QCloseEvent * closeEvent)
{
	hide();
	closeEvent->ignore();
}
