#ifndef DESKTOP_H
#define DESKTOP_H

#include <QSplitter>
#include <QPalette>
#include <QBoxLayout>
#include <QGroupBox>
#include <QVBoxLayout>
#include <QListWidget>
#include <QPixmap>
#include <QLabel>
#include <QStringList>
#include <QCalendarWidget>
#include <QGraphicsDropShadowEffect>
#include <QWindow>
#include <QSharedMemory>
#include <QThread>
#include <QFocusFrame>
#include <QFocusEvent>
#include <QBitmap>
#include <QScrollArea>

#include "constants.h"
#include "settings.h"
#include "application.h"
#include "lwclient.h"
#include "tablewidget.h"
#include "initprogress.h"
#include "filesystemwidget.h"
#include "placeswidget.h"
#include "appswidget.h"
#include "splitter.h"

class Desktop : public PixmapFrame
{
	Q_OBJECT

	public:
		Desktop(InitProgress * initProgress, QWidget * parent, QString color);

		HWND getWindowId();
		void resize (QSize size = QSize(0, 0) );
		void initializeDesktopWidgets();
		void updateDesktopWidgets();

	public slots:
		void setLogo();
		void mousePressed();
		void reloadFavorites();
		void saveGeometry();

	signals:
		void launchApplication(Application * app);
		void launchApplication();
		void selectWidget(QWidget * widget);

	protected:
		void addDesktopWidget(QString iniFile);
		void locateWidget(int widgetId);
		void closeEvent(QCloseEvent * closeEvent);

		AppsWidget * appsWidget;
		QScrollArea * appsScrollArea;
		FileSystemWidget * folderView;
		TableWidget * widgetsWidget;
		PlacesWidget * placesWidget;
		QScrollArea * widgetsScrollArea;
		QWidget * hLine;
		int spacerHeight;
		QSpacerItem * spacer;
		Splitter * splitter;
		InitProgress * initProgress;
		QList<QWidget*> desktopWidgets;
		QList<QSize> desktopWidgetsMinSizes;
		QList<QSize> desktopWidgetsMaxSizes;
		QList<QString> desktopWidgetsVerticalAlignment;
		QList<QString> desktopWidgetsHorizontalAlignment;
		bool useDesktopWidgets;
};

#endif
