#include "displayconfig.h"

#include <X11/Xlib.h>
#include <X11/extensions/Xrandr.h>
#include <QX11Info>

DisplayConfig::DisplayConfig(QWidget *parent ) : QDialog( parent )
{
	setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
	init ();
}

void DisplayConfig::init()
{
	readDisplay();

	layout = new QGridLayout(this);
	layout->setSpacing(10);
	layout->setMargin(20);

	QFont bFont;
	bFont.setFamily( Settings::getUserValue("BoldFont", LIBERUS_FONT_BOLD).toString() );

	monitorsGroup = new QGroupBox;
	QHBoxLayout * monitorsGroupLayout = new QHBoxLayout;

	for ( int i = 0; i < monitorList.count(); i++ )
	{
		monitors.append( new QPushButton( monitorId[i] + " - " + monitorName[i] ) );
		monitorsGroupLayout->addWidget( monitors[i] );
	}

	monitorsGroup->setLayout( monitorsGroupLayout );

	/*QLabel * normalFontLabel = new QLabel(tr("Default font:"), this);
	normalFontLabel->setMinimumWidth( 150 );
	normalFontLabel->setFont( bFont );
	normalFont = new QComboBox( this );
	normalFont->setMinimumWidth( 200 );

	QLabel * boldFontLabel = new QLabel(tr("Bold font:"), this);
	boldFontLabel->setMinimumWidth( 150 );
	boldFontLabel->setFont( bFont );
	boldFont = new QComboBox( this );
	boldFont->setMinimumWidth( 200 );

	QLabel * fontSizeLabel = new QLabel(tr("Default font size:"), this);
	fontSizeLabel->setFont( bFont );
	fontSize = new QSpinBox( this );
	fontSize->setMinimum(6);
	fontSize->setMaximum(18);

	QLabel * appButtonFontSizeLabel = new QLabel(tr("Desktop icons font size:"), this);
	appButtonFontSizeLabel->setFont( bFont );
	appButtonFontSize = new QSpinBox( this );
	appButtonFontSize->setMinimum(6);
	appButtonFontSize->setMaximum(18);
	*/

	acceptButton = new QPushButton( tr("Accept"), this);
	acceptButton->setFlat( true );
	acceptButton->setStyleSheet(STANDARDBUTTONSTYLE);
	acceptButton->setMinimumSize( 100, 24 );
	cancelButton = new QPushButton( tr("Cancel"), this);
	cancelButton->setFlat( true );
	cancelButton->setStyleSheet(STANDARDBUTTONSTYLE);
	cancelButton->setMinimumSize( 100, 24 );

	/*layout->addWidget(normalFontLabel, 1, 0);
	layout->addWidget(normalFont, 1, 1, 1, 5);
	layout->addWidget(boldFontLabel, 2, 0);
	layout->addWidget(boldFont, 2, 1, 1, 5);
	layout->addWidget(fontSizeLabel, 3, 0);
	layout->addWidget(fontSize, 3, 1, 1,5);
	layout->addWidget(appButtonFontSizeLabel, 4, 0);
	layout->addWidget(appButtonFontSize, 4, 1, 1, 5);
	*/
	layout->addWidget(acceptButton, 5, 4);
	layout->addWidget(cancelButton, 5, 5);

	for (int i = 0; i < 8; i++)
	{
		layout->setRowStretch(i, 1);
		layout->setColumnStretch(i, 1);
	}

	connect(acceptButton, SIGNAL(clicked()), this, SLOT(accept()));
	connect(cancelButton, SIGNAL(clicked()), this, SLOT(reject()));

	loadSettings();
}

void DisplayConfig::readDisplay()
{
	primaryMonitorIndex = -1;
	monitorList.clear();
	monitorId.clear();
	monitorActive.clear();
	monitorName.clear();
	
#ifdef Q_OS_LINUX
/*
	XRRScreenResources * screenResources = XRRGetScreenResources(QX11Info::display(), XDefaultRootWindow(QX11Info::display()));

	int ncrtc = screenResources->ncrtc;
	int noutput = screenResources->noutput;

	printf("CRTCs: %d, OUTPUTS: %d\n", ncrtc, noutput);
	for ( int i = 0; i < ncrtc; i++ )
	{
		XRRCrtcInfo *crtcInfo = XRRGetCrtcInfo(QX11Info::display(), screenResources, screenResources->crtcs[i]);
		printf("%dx%d+%d+%d CRT outputs: %d\n", crtcInfo->width, crtcInfo->height, crtcInfo->x, crtcInfo->y, crtcInfo->noutput);

		for ( int j = 0; j < crtcInfo->noutput; j++ )
		{
			XRROutputInfo *outputInfo = XRRGetOutputInfo (QX11Info::display(), screenResources, crtcInfo->outputs[i]);
			printf("CRTC output %d, number of clones: %d\n", j, outputInfo->nclone);
		
			XRRFreeOutputInfo(outputInfo);
		}
		XRRFreeCrtcInfo(crtcInfo);
	}
	for ( int i = 0; i < noutput; i++ )
	{
		XRROutputInfo *outputInfo = XRRGetOutputInfo (QX11Info::display(), screenResources, screenResources->outputs[i]);
		printf("Output: %d, number of clones: %d\n", i, outputInfo->nclone);
		XRRFreeOutputInfo(outputInfo);
	}
	XRRFreeScreenResources(screenResources);
*/

/*
	QProcess * detectProcess = new QProcess();
	detectProcess->start( "xrandr", QStringList("--auto") );
	detectProcess->waitForFinished();
	delete detectProcess;

	int numberOfMonitors = -1;
	QProcess * listProcess = new QProcess();
	listProcess->start( "xrandr", QStringList("--listmonitors") );
	listProcess->waitForFinished();
	listProcess->setReadChannel(QProcess::StandardOutput);
	if ( listProcess->canReadLine() )
	{
		QString line = QString::fromLocal8bit(process->readLine());
		if ( line.startsWith( "Monitors: " ) )
		{
			numberOfMonitors = line.section(":", 1, 1).toInt();
		}
	}

	if ( numberOfMonitors >= 1 )
	{
		int count = 0;
		while ( listProcess->canReadLine() )
		{
			monitorList += QString::fromLocal8bit(process->readLine()).simplified();
			count++;
		}

		if ( numberOfMonitors == count )
		{
			for ( int i = 0; i < count; i++ )
			{
				monitorId += monitorList[i].section(":", 0, 0);
				if ( monitorList[i].section(" ", 1, 1).contains("*") )
				{
					primaryMonitorIndex = i;
				}
				monitorActive +=  monitorList[i].section(" ", 1, 1).contains("+");
				monitorName = monitorList[i].section(" ", 3, 3);
			}
		} else {
			monitorList.clear();
			numberOfMonitors = -1;
		}
	}
	*/
#endif
}

void DisplayConfig::loadSettings()
{
}

void DisplayConfig::saveSettings()
{
	/*
	Settings::setUserValue( "Font", normalFont->currentText() );
	Settings::setUserValue( "BoldFont", boldFont->currentText() );
	Settings::setUserValue( "FontSize", QString::number(fontSize->value()) );
	Settings::setUserValue( "AppButtonFontSize", QString::number(appButtonFontSize->value()) );
	*/
}

void DisplayConfig::accept()
{
	saveSettings();
	setFocus(); // Get out focus from button
}

void DisplayConfig::reject()
{
	loadSettings(); // Set user settings values
	setFocus(); // Get out focus from button
}
