#include "dragdrop.h"

void DragDrop::dragEnterEvent(QDragEnterEvent* event, QString mimeFormat)
{
	if (event->mimeData()->hasFormat(mimeFormat))
	{
		event->accept();
	} else {
		event->ignore();
	}
}

void DragDrop::dragMoveEvent(QDragMoveEvent* e, QString mimeFormat)
{
	if (e->mimeData()->hasFormat(mimeFormat))
	{
		e->setDropAction(Qt::MoveAction);
		e->accept();
	} else {
		e->ignore();
	}
}

QPair<QStringList,QModelIndex> DragDrop::dropEvent(QDropEvent* e, QAbstractItemView * itemView)
{
	QModelIndex target = itemView->indexAt( e->pos() );

	QPair<QList<QFileInfo>,bool> fileList = ClipboardFiles::entries( e->mimeData() );

	if ( target.isValid() )
	{
		QFileInfo targetFileInfo = static_cast<QFileSystemModel*>(itemView->model())->fileInfo( target );
		qDebug("Dropping file '%s'", targetFileInfo.absoluteFilePath().toUtf8().data());
		if ( ! targetFileInfo.isDir() )
		{
			target = itemView->currentIndex();
		}
	} else {
		if ( itemView->rootIndex().isValid() )
		{
			target = itemView->rootIndex();
		}
	}

	QStringList ret;

	if ( target.isValid() )
	{
		for ( int i = 0; i < fileList.first.count(); i++ )
		{
			ret += fileList.first.at(i).absoluteFilePath();
		}
	}

	return QPair<QStringList,QModelIndex>(ret, target);
}

void DragDrop::startDrag(Qt::DropActions supportedActions, QAbstractItemView * itemView, bool fileSystemMode)
{
	QList<QModelIndex> indexes = itemView->selectionModel()->selectedIndexes();
	QByteArray operation;
	if ( supportedActions & Qt::MoveAction )
	{
		operation = "cut";
	} else {
		operation = "copy";
	}
	QMimeData * mimeData = new QMimeData;
	QByteArray data;
	for ( int i = 0; i < indexes.count(); i++ )
	{
		//qDebug("Index %d: %s", i, indexes[i].data().toByteArray().data() );
		if ( fileSystemMode )
		{
			if ( indexes[i].isValid() )
			{
				data = "file://" + static_cast<QFileSystemModel*>(itemView->model())->fileInfo( indexes[i] ).absoluteFilePath().toUtf8();
				qDebug("startDrag data: '%s'", data.data());
			}
		} else {
			if ( indexes[i].isValid() )
			{
				data = itemView->model()->data( indexes[i] ).toByteArray();
			}
		}

		if ( i != indexes.count() - 1 )
		{
			//data += "\n";
		}
		mimeData->setData( "text/plain", data );
		data += "\n";
		mimeData->setData( "text/uri-list", data );
#if defined(Q_OS_LINUX) || defined(Q_OS_FREEBSD)
		data = operation + "\n" + data + "\0";
		QStringList formattedMimes = QString( FILE_MIMETYPE_FORMAT_LIST ).split(',');
		for ( int i = 0; i < formattedMimes.count(); i++ )
		{
			mimeData->setData( formattedMimes[i], data );
			qDebug("startDrag setting data '%s' with format '%s'", data.data(), formattedMimes[i].toUtf8().data());
		}
#endif
	}

	qDebug("Drag supported actions: %04x", static_cast<unsigned int>(supportedActions));

	if ( ! data.isEmpty() )
	{
		QIcon icon = itemView->model()->itemData( indexes[0] )[ Qt::DecorationRole ].value<QIcon>();
		QDrag * drag = new QDrag(itemView);
		drag->setMimeData( mimeData );
		if ( supportedActions & Qt::MoveAction || supportedActions & Qt::CopyAction )
		{
			drag->setDragCursor( icon.pixmap( ICON_SIZE ), Qt::MoveAction );
			drag->exec(Qt::MoveAction | Qt::CopyAction, Qt::MoveAction);
		} else {
			qDebug("Not supported action!");
		}
	}
	/* mimeData must not be deleted, it will be deleted when needed */
}
