#include "fileiconprovider.h"

FileIconProvider::FileIconProvider() : QFileIconProvider()
{
	previewImages = true;
}

QIcon FileIconProvider::icon ( IconType iconType) const
{
    switch( iconType )
    {
        case QFileIconProvider::Folder:
            return QIcon( ICON_DIR_64 + QString(QDir::separator()) + "folder.png" );
        break;
        default:
            return QIcon( ICON_DIR_64 + QString(QDir::separator()) + "openwith.png" );
    }
}
QIcon FileIconProvider::icon ( const QFileInfo & info ) const
{
	if ( info.isDir() )
	{
        return QIcon( ICON_DIR_64 + QString(QDir::separator()) + "folder.png" );
	}

	QIcon icon;

	if ( previewImages )
	{
		if ( info.size() <= MAXPREVIEWSIZE )
		{
			QByteArray imageFormat = QImageReader::imageFormat( info.absoluteFilePath() );
			if ( ! imageFormat.isEmpty() )
			{
				icon = QIcon( QPixmap( info.absoluteFilePath() ) );
			}
		}
	}

	if ( icon.isNull() )
	{
		icon = Application::getIcon( info );
	}

	if (icon.isNull())
    {
        return QIcon( ICON_DIR_64 + QString(QDir::separator()) + "openwith.png" );
        //return QFileIconProvider::icon(info);
    } else {
		return icon;
    }
}

void FileIconProvider::setPreviewImages(bool enable)
{
	previewImages = enable;
}

bool FileIconProvider::getPreviewImages()
{
	return previewImages;
}
