#ifndef FILETHREAD_H
#define FILETHREAD_H

#include <QThread>
#include <QObject>
#include <QFile>
#include <QFileInfo>
#include <QStorageInfo>
#include <QMutex>
#include <QWaitCondition>
#include <QDir>
#include <QMessageBox>
#include <QProcess>

#include "lwclient.h"

#ifdef Q_OS_WIN
//extern Q_CORE_EXPORT int qt_ntfs_permission_lookup;
#endif

class FileThread : public QThread
{
	Q_OBJECT

public:
	enum FileOperation { CALCULATE_SIZES, COPY_FILES, REMOVE_FILES, MOVE_FILES, REMOTE_COPY };
	enum FileError { CANNOT_READ_FILE, CANNOT_WRITE_FILE, CANNOT_CREATE_DIR, CANNOT_MOVE_FILES, FILE_EXISTS, NOT_ENOUGH_SPACE, INSUFFICIENT_PERMISSIONS };

	FileThread(FileThread::FileOperation operation, QObject * parent, QFileInfoList sourceFiles, QDir targetDir = QDir());
	FileThread(FileThread::FileOperation operation, QObject * parent, QFileInfo sourceFile, QDir targetDir = QDir());
	FileThread(FileThread::FileOperation operation, QObject * parent, QStringList sourceFiles, QString targetDir);

	void setUserResponse(QMessageBox::StandardButton response);
	void waitUserResponse();
	long long sizes();
	int files();
	int dirs();
	void setOperation(FileOperation operation);
	FileOperation operation();
	void run();

public slots:
	void finishThread();

signals:
	void changedCurrentOperation(FileThread::FileOperation operation);
	void finished();
	void finished(QStringList fileNames,QString targetDir);
	void calculateProgress(long long size, int files, int dirs);
	void copyProgress(QFileInfo file, long long totalSize, long long totalCopied, long long currentFileSize, long long currentFileCopied);
	void currentFile(QString file);
	void error(FileThread::FileError errorType, QStringList files);

private:
	void getDirSize(QFileInfo file);
	void removeFile(QFileInfo file);
	void moveFile(QFileInfo sourceFile, QDir targetDir);
	void copyFile(QFileInfo sourceFile, QDir targetDir, bool move = false);
	void copyFilesToProfilesServer(QStringList fileNames, QString targetDir);

	QFileInfo _sourceFile;
	QFileInfoList _sourceFiles;
	QStringList _sourceFilesStringList;
	QDir _targetDir;
	QString _targetDirString;
	long long _sizes;
	long long _copied;
	int _files;
	int _dirs;
	bool _isSameFileSystem;
	QMutex mutex;
	QMutex finishMutex;
	QMutex waitMutex;
	QWaitCondition waitUser;
	bool _terminate;
	bool _pause;
	FileOperation _operation;

	bool askToReplace;
	bool replace;
};

Q_DECLARE_METATYPE(FileThread::FileOperation);
Q_DECLARE_METATYPE(FileThread::FileError);

#endif // FILETHREAD_H
