#include "iconviewitemdelegate.h"

IconViewItemDelegate::IconViewItemDelegate( QObject * parent ) : QItemDelegate( parent )
{
	this->pressedPos = QPoint(-1, -1);
	this->selectionShadow = false;
}

void IconViewItemDelegate::drawDisplay(QPainter *painter, const QStyleOptionViewItem &option, const QRect &rect, const QString &text) const
{
	Q_UNUSED( option );
	QRect newRect( rect.x(), LISTVIEWITEM_TOP_MARGIN + rect.y() -  2 * LISTVIEWITEM_FONT_SIZE, rect.width(), rect.height() + 2 * LISTVIEWITEM_FONT_SIZE );
	QStringList textList = text.split(" ");
	QStringList newTextList;
	QString addString;
	int lines = 0;
	for ( int i = 0; i < textList.count() && lines < 2; i++ )
	{
		if ( addString.isEmpty() )
		{
			addString = textList[i];
		}
		if ( (i + 1) < textList.count() )
		{
			QString newAddString = addString + " " + textList[i+1];
			if ( newAddString == painter->fontMetrics().elidedText(newAddString, Qt::ElideRight, rect.width() )  || lines == 1 )
			{
				addString = newAddString;
				if ( ( i + 1 ) == textList.count() )
				{
					newTextList += painter->fontMetrics().elidedText(addString, Qt::ElideRight, rect.width());
				}
			} else {
				newTextList += painter->fontMetrics().elidedText(addString, Qt::ElideRight, rect.width());
				addString.clear();
				lines++;
			}
		} else {
			newTextList += painter->fontMetrics().elidedText(addString, Qt::ElideRight, rect.width() );
			addString.clear();
			lines++;
		}
	}
	QString newText = newTextList.join("\n");
	painter->drawText( newRect, Qt::AlignHCenter | Qt::TextWordWrap, newText);
}

void IconViewItemDelegate::drawFocus(QPainter *painter, const QStyleOptionViewItem &option, const QRect &rect) const
{
	QRect newRect( rect.x(), LISTVIEWITEM_TOP_MARGIN + rect.y() - 2 * LISTVIEWITEM_FONT_SIZE, rect.width(), rect.height() + 2 * LISTVIEWITEM_FONT_SIZE );
	QItemDelegate::drawFocus( painter, option, newRect );
}

void IconViewItemDelegate::drawDecoration(QPainter *painter, const QStyleOptionViewItem &option, const QRect &rect, const QPixmap &pixmap) const
{
	QPixmap newPixmap = pixmap;
	QRect newRect = rect;
	if ( option.state & QStyle::State_MouseOver ) // Hover
	{
		newPixmap = IconViewItemDelegate::toGrayScale( pixmap );
	}
	if ( rect.contains(pressedPos) )
	{
		int diff = 6;
		newRect.setLeft( rect.x() + diff );
		newRect.setTop( LISTVIEWITEM_TOP_MARGIN + rect.y() + diff );
		newRect.setWidth( rect.width() - diff );
		newRect.setHeight( rect.height() - diff );
	} else {
		newRect.setTop( LISTVIEWITEM_TOP_MARGIN + rect.y() );
		newRect.setHeight( rect.height() );
	}
	painter->drawPixmap( newRect, newPixmap );
}

void IconViewItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
	QRect rect = option.rect;
	if ( option.state & QStyle::State_Selected && selectionShadow ) // Selected
	{
		painter->fillRect( rect, QColor(0, 0, 0, 20) );
	}
	QItemDelegate::paint( painter, option, index );
}

bool IconViewItemDelegate::editorEvent(QEvent *event, QAbstractItemModel *model, const QStyleOptionViewItem &option, const QModelIndex &index)
{
	return QItemDelegate::editorEvent( event, model, option, index );
}

QSize IconViewItemDelegate::sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const
{
	Q_UNUSED( option);
	Q_UNUSED( index );
	return LISTVIEWITEM_GRID_SIZE;
}

void IconViewItemDelegate::setPressed(QPoint pos)
{
	pressedPos = pos;
}

void IconViewItemDelegate::setSelectionShadow(bool selectionShadow)
{
	this->selectionShadow = selectionShadow;
}

QPixmap IconViewItemDelegate::toGrayScale(QPixmap pixmap)
{
	QImage sourceImage = pixmap.toImage();
	QImage targetImage( sourceImage.size(), QImage::Format_RGBA8888 );
	for ( int x = 0; x < sourceImage.width(); x++ )
	{
		for ( int y = 0; y < sourceImage.height(); y++ )
		{
			QRgb sourcePixel = sourceImage.pixel(x, y);
			int grayPixel = qGray(sourcePixel);
			QRgb targetPixel = qRgba(grayPixel, grayPixel, grayPixel, sourceImage.alphaChannel().pixel( x, y ));
			targetImage.setPixel(x, y, targetPixel);
		}
	}
	return QPixmap::fromImage( targetImage );
}

QPixmap IconViewItemDelegate::highlight(QPixmap pixmap)
{
	QPixmap highlighedPixmap = pixmap;
	QPainter pixmapPainter;
	pixmapPainter.begin(&highlighedPixmap);
	pixmapPainter.setCompositionMode(QPainter::CompositionMode_SourceAtop);
	pixmapPainter.fillRect(highlighedPixmap.rect(), QColor(180, 180, 180, 100));
	pixmapPainter.end();
	return highlighedPixmap;
}

QPixmap IconViewItemDelegate::fillBackground(QPixmap pixmap, QColor color)
{
	QPixmap filledPixmap = pixmap;
	QPainter pixmapPainter;
	pixmapPainter.begin(&filledPixmap);
	pixmapPainter.fillRect(filledPixmap.rect(), color);
	pixmapPainter.drawPixmap(0, 0, pixmap.width(), pixmap.height(), pixmap);
	pixmapPainter.end();
	return filledPixmap;
}
