#include "languageconfig.h"

LanguageConfig::LanguageConfig(QWidget *parent ) : QDialog( parent )
{
	setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
	init ();
}

void LanguageConfig::init()
{
	layout = new QGridLayout(this);
	layout->setSpacing(10);
	layout->setMargin(20);

	QFont bFont;
	bFont.setFamily( Settings::getUserValue("BoldFont", LIBERUS_FONT_BOLD).toString() );

	QLabel * languageLabel = new QLabel(tr("Language:"), this);
	languageLabel->setMinimumWidth( 150 );
	languageLabel->setFont( bFont );
	language = new QComboBox( this );
	language->setMinimumWidth( 200 );

#ifdef Q_OS_LINUX
	QLabel * keyboardLayoutLabel = new QLabel(tr("Keyboard layout:"), this);
	keyboardLayoutLabel->setMinimumWidth( 150 );
	keyboardLayoutLabel->setFont( bFont );
	keyboardLayout = new QComboBox( this );
	keyboardLayout->setMinimumWidth( 200 );
#endif

	acceptButton = new QPushButton( tr("Accept"), this);
	acceptButton->setFlat( true );
	acceptButton->setStyleSheet(STANDARDBUTTONSTYLE);
	acceptButton->setMinimumSize( 100, 24 );
	cancelButton = new QPushButton( tr("Cancel"), this);
	cancelButton->setFlat( true );
	cancelButton->setStyleSheet(STANDARDBUTTONSTYLE);
	cancelButton->setMinimumSize( 100, 24 );

	layout->addWidget(languageLabel, 1, 0);
	layout->addWidget(language, 1, 1, 1, 5);
#ifdef Q_OS_LINUX
	layout->addWidget(keyboardLayoutLabel, 2, 0);
	layout->addWidget(keyboardLayout, 2, 1, 1, 5);
#endif
	layout->addWidget(acceptButton, 5, 4);
	layout->addWidget(cancelButton, 5, 5);

	for (int i = 0; i < 8; i++)
	{
		layout->setRowStretch(i, 1);
		layout->setColumnStretch(i, 1);
	}

	connect(acceptButton, SIGNAL(clicked()), this, SLOT(accept()));
	connect(cancelButton, SIGNAL(clicked()), this, SLOT(reject()));

	loadSettings();
}

void LanguageConfig::loadSettings()
{
	QStringList languageIds;
	languageIds << "es" << "en";
	QStringList languages;
	languages << tr("Spanish") << tr("English");

	int index = 0;
	QString lang  = Settings::getUserValue("Language", "es").toString();
	for ( int i = 0; i < languages.count(); i++ )
	{
		language->addItem( languages[i], languageIds[i] );
		if ( languageIds[i] == lang )
		{
			index = i;
		}
	}
	language->setCurrentIndex( index );
#ifdef Q_OS_LINUX
	QStringList keyboardLayoutIds;
	keyboardLayoutIds << "es" << "us" << "gb" << "de" << "fr";

	QStringList keyboardLayouts;
	keyboardLayouts << tr("Spanish") << tr("English (US)") << tr("English (UK)") << tr("German") << tr("French");

	index = 0;
	QString keyb  = Settings::getUserValue("KeyboardLayout", "es").toString();
	for ( int i = 0; i < keyboardLayouts.count(); i++ )
	{
		keyboardLayout->addItem( keyboardLayouts[i], keyboardLayoutIds[i] );
		if ( keyboardLayoutIds[i] == keyb )
		{
			index = i;
		}
	}

	keyboardLayout->setCurrentIndex( index );
#endif
}

void LanguageConfig::saveSettings()
{
	Settings::setUserValue( "Language", language->currentData() );
#ifdef Q_OS_LINUX
	Settings::setUserValue( "KeyboardLayout", keyboardLayout->currentData() );
	QString program("setxkbmap");
	QStringList arguments;
	arguments += "-layout";
	arguments += keyboardLayout->currentData().toString();
	QProcess::execute( program, arguments );
#endif
	QTranslator translator;
	translator.load("liberus-desktop_" + language->currentData().toString() );
	qApp->installTranslator( & translator );
#ifdef Q_OS_LINUX
	QString lang = LWLocale::getLang( language->currentData().toString().toUtf8() );
	qputenv("LANG", lang.toUtf8());
#endif

	QMessageBox::information( this, tr("Restart needed"), tr("The session must be restarted in order to implement all changes") );
}

void LanguageConfig::accept()
{
	saveSettings();
	setFocus(); // Get out focus from button
}

void LanguageConfig::reject()
{
	loadSettings(); // Set user settings values
	setFocus(); // Get out focus from button
}
