#ifndef MANAGER_H
#define MANAGER_H

#include <QTextStream>

#include <QApplication>

#include <QTabWidget>
#include <QPushButton>
#include <QIcon>
#include <QProcess>
#include <QAbstractEventDispatcher>
#include <QVBoxLayout>
#include <QList>
#include <QTimer>
#include <QSize>
#include <QPoint>
#include <QTimer>
#include <QMovie>
#include <QLabel>
#include <QWindow>
#include <QGuiApplication>
#include <QMap>
#include <QFileDialog>
#include <QMessageBox>
#include <QFile>
#include <QFileSystemWatcher>
#include <QThread>

#include "constants.h"
#include "tabbar.h"
#include "settings.h"
#include "lwclient.h"
#include "appserver.h"
#include "sessionmanager.h"
#include "threadbutton.h"
#include "initscreen.h"
#include "initprogress.h"
#include "winmanager.h"
#include "winaction.h"
#include "power.h"

#ifdef Q_OS_LINUX
#include <sys/types.h>
#include <signal.h>
#include <errno.h>
#endif
#ifdef Q_OS_WIN
//#include <WtsApi32.h>
#endif
class WinManager;
class TabBar;

Q_DECLARE_METATYPE( HWND )

class Manager : public QObject
{
	Q_OBJECT

	public:
		Manager(InitProgress * initProgress, QString desktopUsername = QString(), QString desktopPassword = QString(), QString serverAddress = QString(), int serverPort = -1, QWidget *parent = 0);
		~Manager();

		static QSize getDesktopSize();
		static void startWindowManager( bool disableAltTab = false );
		static void stopWindowManager();

		static void generateMimeAppsList();
		static void launchRemoteApplication(QString fileName, QString targetDir, QString remoteApplication = QString());
		static void openRemoteServerDir(QString targetDir);

	public slots:
		void init();
		void exitApp();

	private slots:
		void launchApplication(Application * app);
		void timeout();
		void resize();
		void clientConnected();
		void loadedFinished();
		static bool connectToServer(QString serverAddress, int serverPort);
		void mapWindow(HWND id);
		void mapWidget(QWidget * widget);
		void setDockWindow();

	private:
		void createDesktopLinks(QString homeDir);

		SessionManager * sessionManager;
		WinManager * winManager;
		QWidget * parent;
		QFileSystemWatcher * appFileWatcher;
		InitProgress * initProgress;

		QList<Q_PID> pids;

		TabBar * tabBar;

		static QProcess * wmProcess;

		QTimer timer;
		QString launchingRemote;
		QString serverAddress;
		int serverPort;
};

#endif
