#include "pixmapframe.h"

PixmapFrame::PixmapFrame(QWidget * parent, QPixmap pixmap) : QFrame( parent )
{
	_opacity = 1.0;
	this->pixmap = pixmap;
	this->expanded = true;
	this->centered = true;
	x = 0;
	y = 0;

	textPosX = 0;
	textPosY = 0;
	timeTextPixelSize = 70;
	dateTextPixelSize = 45;
	fontFamily = "Arial";
	fontColor = QColor( 255, 255, 255 );

	enableText = true;

	setMouseTracking( true );
}

qreal PixmapFrame::opacity()
{
	return _opacity;
}

void PixmapFrame::setOpacity(qreal opacity)
{
	_opacity = opacity;
	repaint();
}

void PixmapFrame::setColor( QColor color )
{
	_color = color;
	setStyleSheet(QString("background-color: rgb(%1, %2, %3);").arg(color.red()).arg(color.green()).arg(color.blue()));
}

QColor PixmapFrame::color()
{
	return _color;
}

void PixmapFrame::showText(bool enable)
{
	enableText = enable;
	repaint();
}

void PixmapFrame::setPixmap(QPixmap pixmap, bool centered, bool expanded )
{
	this->pixmap = pixmap;
	this->centered = centered;
	this->expanded = expanded;
	resize();
}

void PixmapFrame::setText( QString timeText, QString dateText )
{
	this->timeText = timeText;
	this->dateText = dateText;
	repaint();
}

void PixmapFrame::setTextPos( int posX, int posY )
{
	textPosX = posX;
	textPosY = posY;
}

void PixmapFrame::setFontFamily( QString fontFamily )
{
	this->fontFamily = fontFamily;
}

void PixmapFrame::setTimeFontPixelSize( int pixelSize )
{
	this->timeTextPixelSize = pixelSize;
}

void PixmapFrame::setDateFontPixelSize( int pixelSize )
{
	this->dateTextPixelSize = pixelSize;
}

void PixmapFrame::setFontColor( QColor fontColor )
{
	this->fontColor = fontColor;
}

void PixmapFrame::resize()
{
	if ( this->expanded )
	{
		x = 0;
		y = 0;

		if ( (double)pixmap.width() / (double)width() > (double)pixmap.height() / (double)height() )
		{
			this->pixmap = pixmap.scaledToHeight(height(), Qt::SmoothTransformation);
		} else {
			this->pixmap = pixmap.scaledToWidth(width(), Qt::SmoothTransformation);
		}
	} else {
		this->pixmap = pixmap;
		if (centered)
		{
			x = (width() - pixmap.width() ) / 2;
			y = (height() - pixmap.height() ) / 2;
		} else {
			x = width() - pixmap.width();
			y = height() - pixmap.height();
		}
	}
	repaint();
}

void PixmapFrame::paintEvent ( QPaintEvent * )
{
	QPainter painter(this);

	painter.drawPixmap(x, y, pixmap);
	painter.setOpacity(1.0 - _opacity); // Opacity refers to pixmap, not to background
	painter.fillRect( geometry(), _color );

	if ( _opacity == 1.0 )
	{
		painter.setOpacity( 1.0 );
	} else {
		painter.setOpacity( (_opacity - LOGINSCREEN_OBSCURED_OPACITY) * ( 1.0 / (1.0 - LOGINSCREEN_OBSCURED_OPACITY) )  );
	}

	if ( enableText )
	{
		QFont font( fontFamily );
		font.setPixelSize( timeTextPixelSize );
		painter.setFont( font );
		painter.setPen( fontColor );
		painter.drawText(textPosX, textPosY, timeText);
		font.setPixelSize( dateTextPixelSize );
		painter.setFont( font );
		painter.drawText(textPosX, textPosY + timeTextPixelSize, dateText);
	}

	painter.end();
}

void PixmapFrame::mousePressEvent(QMouseEvent *)
{
	emit mousePressed();
}

void PixmapFrame::mouseMoveEvent(QMouseEvent *)
{
	emit mouseMoved();
}

void PixmapFrame::keyPressEvent(QKeyEvent *event)
{
	if ( event->key() == Qt::Key_Escape )
	{
		emit escapePressed();
	} else {
		QFrame::keyPressEvent( event );
		emit keyPressed( event );
	}
}
