#include "remoteappserver.h"

#include <QDir>

RemoteAppServer::RemoteAppServer(QObject *parent) : QObject(parent)
{
	isAuthenticated = false;
}

RemoteAppServer::~RemoteAppServer()
{
	//sendPortToServer(0); // unset address and port for this user
}

int RemoteAppServer::startServer()
{
	LWClient * lwClient = new LWClient();
	if ( ! lwClient->connect( Settings::getServerAddress(), Settings::getServerPort() ) )
	{
		Settings::logError("Cannot connect to Liberus Server");
		delete lwClient;
		return -1;
	}

	/*if ( getToken().isEmpty() )
	{
		Settings::logError("Cannot find user token");
		lwClient->close();
		delete lwClient;
		return -2;
	}

	if ( ! lwClient->authenticate( Settings::getFullUserName(), token ) )
	{
		Settings::logError("Failed token authentication on Liberus Server");
		lwClient->close();
		delete lwClient;
		return -3;
	}*/

	QString thisIpAddress;
	const QHostAddress &localhost = QHostAddress(QHostAddress::LocalHost);
	for (const QHostAddress &address: QNetworkInterface::allAddresses())
	{
		if (address.protocol() == QAbstractSocket::IPv4Protocol && address != localhost)
		{
			thisIpAddress = address.toString();
		}
	}

	if ( thisIpAddress.isEmpty() )
	{
		Settings::logError("Cannot fetch own IPv4 address");
		lwClient->close();
		delete lwClient;
		return -4;
	}

	QStringList parameters;
	parameters += thisIpAddress;
	parameters += Settings::getFullUserName();
	QString ret = lwClient->sendMessage( "_serverCommand_set_remote_server", parameters );
	if ( ret.startsWith("Error") )
	{
		Settings::logError( QString( "Failed to negotiate set remote server with Liberus Server: %1" ).arg( ret ) );
		lwClient->close();
		delete lwClient;
		return -5;
	}

	LWSocket::setSocketBlocking( lwClient->socket() );
	while ( true )
	{
		QString message = LWSocket::receiveMessage( lwClient->socket(), MAX_MESSAGE_SIZE).simplified();
		Settings::logInfo( QString( "Received message: %1" ).arg( message ) );
		QString ret = doResponse(message);
		Settings::logInfo( QString( "Sending response: %1" ).arg( ret ) );
		if ( ! LWSocket::sendMessage( lwClient->socket(), ret ) )
		{
			break;
		}

		if ( message == "_serverCommand_close" )
		{
			break;
		}
	}

	lwClient->close();
	delete lwClient;
	return 0;

	/*SOCKET socket = LWSocket::createServerSocket( "0.0.0.0", 0, 1 ); // port 0, bind will give next available port
	if ( socket < 0 )
	{
		qDebug("Something happened creating server socket. Exiting with error code %d", socket);
		return socket;
	} else {
		qDebug("Socket created");
	}

	struct sockaddr_in sin;
	socklen_t len = sizeof(sin);
	if (getsockname(socket, (struct sockaddr *)&sin, &len) == -1)
	{
		qDebug("Cannot get socket port");
		return -1;
	} else {
		port = ntohs(sin.sin_port);
	}

	qDebug("Serving on port %d", port);

	if ( ! sendPortToServer( port ) )
	{
		qDebug("Cannot send server port %d to Liberus Principal server", port);
		return -2;
	}


	qDebug("Waiting for connection...");
	if ( ! LWSocket::waitForConnection( socket ) )
	{
		qDebug("Something happened waiting for a client connection. Exiting");
		return -1;
	}


	SOCKET clientSocket = LWSocket::acceptClientConnection( socket );

	while ( true  )
	{
		qDebug("Waiting for message...");

		QString message = LWSocket::receiveMessage( clientSocket ).simplified();
		qDebug("Accepted message '%s'", message.toUtf8().data());

		if ( message == "quit" || message.size() == 0 )
		{
			LWSocket::closeSocket( clientSocket );
			break;
		} else {
			if ( ! LWSocket::sendMessage( clientSocket, doResponse(message) ) )
			{
				qDebug("Cannot send response");
				break;
			}
		}
	}

	LWSocket::closeSocket( socket );
	return 0;*/
}

QString RemoteAppServer::doResponse(QString question)
{
	QString command = question.trimmed().section('?', 0, 0);

	QStringList parameters = question.trimmed().section('?', 1, 1).split('&');

	if ( parameters.count() == 1 && parameters[0].isNull() )
		parameters.removeFirst();

	bool badParameters = false;

	if ( command == "_serverCommand_close" )
	{
		if (parameters.count() == 0)
			return "True";
		else
			badParameters = true;
	}

	if ( command == "_serverCommand_open_file" ) // fileName, remoteApplication
	{
		if (parameters.count() == 2)
			return openFile( parameters[0], parameters[1] );
		else
			badParameters = true;
	}

	if ( command == "_serverCommand_open_dir" ) // fileName
	{
		if (parameters.count() == 1)
			return openFile( parameters[0] );
		else
			badParameters = true;
	}

	if ( badParameters )
	{
		return QString("Error: bad number of parameters");
	} else {
		return QString("Error: Unknown command ('%1')").arg(command);
	}
}

QString RemoteAppServer::authenticate(QString username, QString token)
{
	if ( username == Settings::getFullUserName() && token == getToken() )
	{
		isAuthenticated = true;
		return QString("Authentication Ok");
	} else {
		return QString("Error: Bad authentication");
	}
}

QString RemoteAppServer::openFile(QString fileName, QString remoteApplication)
{
	/*if ( ! isAuthenticated )
	{
		return QString("Error: not authenticated");
	}*/
	if ( remoteApplication.isEmpty() )
	{
		QDesktopServices::openUrl( QUrl::fromLocalFile( fileName ) );
	} else {
		QProcess::startDetached( remoteApplication, QStringList(fileName) );
	}
	return fileName;
}

QString RemoteAppServer::getToken()
{
	if ( token.isEmpty() )
	{
		if ( QFile::exists( TOKEN_FILE ) )
		{
			token = Settings::getValue( TOKEN_FILE, "UserToken");
	#ifdef Q_OS_WIN
			if ( token.isEmpty() )
			{
				QString sharedTokenFile = qgetenv("LOGONSERVER") + "\\\\" + Settings::getFullUserName() + "\\\\liberus.ini";
				token = Settings::getValue( sharedTokenFile, "UserToken" );
			}
	#endif
		}
	}
	return token;
}

/*bool RemoteAppServer::sendPortToServer(int port)
{
	LWClient * lwClient = new LWClient();
	if ( ! lwClient->connect( Settings::getServerAddress(), Settings::getServerPort() ) )
	{
		qDebug("Cannot connect to Liberus Server");
		delete lwClient;
		return false;
	}

	if ( getToken().isEmpty() )
	{
		qDebug("Cannot find user token");
		lwClient->close();
		delete lwClient;
		return false;
	}

	if ( ! lwClient->authenticate( Settings::getFullUserName(), token ) )
	{
		qDebug("Failed token authentication on Liberus Server");
		lwClient->close();
		delete lwClient;
		return false;
	}

	QString thisIpAddress;
	if ( port > 0 )
	{
		const QHostAddress &localhost = QHostAddress(QHostAddress::LocalHost);
		for (const QHostAddress &address: QNetworkInterface::allAddresses())
		{
			if (address.protocol() == QAbstractSocket::IPv4Protocol && address != localhost)
			{
				thisIpAddress = address.toString();
			}
		}

		if ( thisIpAddress.isEmpty() )
		{
			qDebug("Cannot fetch local IP address");
			lwClient->close();
			delete lwClient;
			return false;
		}

		QStringList parameters;
		parameters += thisIpAddress;
		parameters += QString::number( port );
		if ( lwClient->sendMessage("set_remote_app_server", parameters).startsWith("Error") )
		{
			qDebug("Cannot send port number to Liberus Server");
			lwClient->close();
			delete lwClient;
			return false;
		}
	} else {
		QString ret = lwClient->sendMessage("unset_remote_app_server");
		if ( ret.startsWith("Error") )
		{
			qDebug("Cannot unset remote app server for this user");
			lwClient->close();
			delete lwClient;
			return false;
		}
	}

	lwClient->close();
	delete lwClient;

	return true;
}*/
