#include "remoteclientmanager.h"

RemoteClientManager::RemoteClientManager()
{
}

void RemoteClientManager::start()
{
	QHash<QString,QString> assignedDrives;
	QHash<QString,QString> sharedFoldersAssignedDrives;
	if ( LWClient::instance()->isConnected() )
	{
		QString fullUserName = Settings::getFullUserName();
		qDebug("Getting redirected drives...");
		QStringList redirectedDrives = LWClient::instance()->sendMessage(QString("get_redirect_drives?%1").arg(fullUserName)).split(",");
		Settings::logInfo(QString("get_redirect_drives output: %1").arg(redirectedDrives.join(",")));
		for (int i = 0; i < redirectedDrives.count(); i++)
		{
			QString drive = redirectedDrives[i].trimmed().section('=', 0, 0).remove("\\").toUpper();
			QString name = redirectedDrives[i].trimmed().section('=', 1, 1);
			if ( name.startsWith("\\\\") ) // Shared folder
			{
				sharedFoldersAssignedDrives[name] = drive;
				Settings::logInfo(QString("Shared folder name: %1 drive: %2").arg(name).arg(drive));
			} else { // Client drive redirect
				name.remove(":").remove("/").remove("\\");
				assignedDrives[name.toLower()] = drive;
				Settings::logInfo(QString("Client redirect name: %1 drive: %2").arg(name).arg(drive));
			}
		}
		names.clear();
		drives.clear();
		remove.clear();
		setRemoteDrives(&names, &drives, &remove, assignedDrives);
		setSharedFoldersDrives(&names, &drives, &remove, sharedFoldersAssignedDrives);
		if ( Settings::homePath().startsWith("\\\\") ) // Home path in shared folder
		{
			setHomeDrive(&names, &drives, &remove);
		}
		emit update();
	}
}

void RemoteClientManager::stop()
{
    for (int i = 0; i < drives.count(); i++)
    {
        if ( remove[i] )
        {
			removeRemoteDrive( drives[i] );
        }
    }
}

QStringList RemoteClientManager::getUsedLogicalDrives()
{
	QStringList used;
#ifdef Q_OS_WIN
	DWORD dwSize = MAX_PATH;
	LPWSTR szLogicalDrives = new WCHAR[MAX_PATH]; // = {0};
	DWORD dwResult = GetLogicalDriveStrings(dwSize, szLogicalDrives);

	if (dwResult > 0 && dwResult <= MAX_PATH)
	{
		LPWSTR szSingleDrive = szLogicalDrives;
		while(*szSingleDrive)
		{
			QString drive = QString::fromWCharArray( szSingleDrive );
			used += drive;
			szSingleDrive += 4;
		}
	}
	delete szLogicalDrives;
#endif
	return used;
}

QString RemoteClientManager::getAvailableLogicalDrive()
{
	QStringList used = getUsedLogicalDrives();
	for ( char letter = 'E'; letter <= 'Z'; letter++ )
	{
		QString drive = QChar(letter) + QString(":\\");
		if (! used.contains(drive))
		{
			return QChar(letter) + QString(":");
		}
	}
	return QString();
}

void RemoteClientManager::removeRemoteDrive(QString drive)
{
#ifdef Q_OS_WIN
	qDebug("Removing drive %s", drive.toUtf8().data());
	WNetCancelConnection2((const wchar_t*)drive.utf16(), 0, TRUE);
#endif
}

QString RemoteClientManager::appendRemoteDrive(QString name, QString _drive)
{
	QString drive;
	if ( _drive.isEmpty() )
	{
		drive = getAvailableLogicalDrive();
	} else {
		if ( getUsedLogicalDrives().contains( _drive ) )
		{
			return _drive;
			//drive = getAvailableLogicalDrive();
		} else {
			drive = _drive;
		}
	}

	QString fullName;
	if ( name.startsWith("\\\\") ) // Shared folder
	{
		fullName = name;
	} else { // Client drive redirect
		fullName = "\\\\tsclient\\" + name;
	}

#ifdef Q_OS_WIN
	NETRESOURCE NetResource;
	memset(&NetResource, 0, sizeof(NetResource));
	NetResource.lpProvider		= nullptr ;
	NetResource.dwType			= RESOURCETYPE_ANY;
	NetResource.lpLocalName = (LPTSTR) drive.utf16();
	NetResource.lpRemoteName = (LPTSTR) fullName.utf16();

	qDebug("name: %ls, drive: %ls\n", NetResource.lpRemoteName, NetResource.lpLocalName );
	Settings::logInfo( QString("Name: %1 Drive: %2").arg(name).arg(drive) );
	if ( WNetAddConnection2(&NetResource, nullptr, nullptr, FALSE) != NO_ERROR )
	{
		qDebug("Something failed connecting %s to local drive %s", name.toUtf8().data(), drive.toUtf8().data());
		Settings::logError( QString("Something failed connecting Name: %1 Drive: %2").arg(name).arg(drive) );
		return QString();
	}
#endif
	return drive;
}

void RemoteClientManager::setHomeDrive(QStringList * names, QStringList * drives, QList<bool> *remove)
{
	if ( ! names->contains( Settings::homePath(), Qt::CaseInsensitive ) )
	{
		QString drive = appendRemoteDrive( Settings::homePath() );

		if ( ! drive.isEmpty() )
		{
			names->append( Settings::homePath() );
			drives->append( drive );
			remove->append( true );
			Settings::setHomePath( drive );
		}
	}
}

void RemoteClientManager::setSharedFoldersDrives(QStringList * names, QStringList * drives, QList<bool> *remove, QHash<QString, QString> assignedDrives)
{
	QHashIterator<QString,QString> it(assignedDrives);
	while (it.hasNext())
	{
		it.next();
		if ( ! names->contains( it.key(), Qt::CaseInsensitive) )
		{
			QString drive = appendRemoteDrive( it.key(), it.value() );

			if ( ! drive.isEmpty() )
			{
				names->append(it.key());
				drives->append(it.value());
				remove->append( true );
			}
		}
	}
}

void RemoteClientManager::setRemoteDrives(QStringList * names, QStringList * drives, QList<bool> *remove, QHash<QString, QString> assignedDrives)
{
#ifdef Q_OS_WIN
	HANDLE hEnum;
	WNetOpenEnum(RESOURCE_CONNECTED, RESOURCETYPE_DISK, 0, NULL, &hEnum);

	DWORD cbBuffer = 1048576;
	void *buffer = LocalAlloc(LMEM_FIXED, cbBuffer);
	LPNETRESOURCE pnr = (LPNETRESOURCE)buffer;

	DWORD err;
	do
	{
		DWORD cEntries = INFINITE;
		DWORD cb = cbBuffer;
		err = WNetEnumResource(hEnum,(LPDWORD) &cEntries, buffer, (LPDWORD)&cb);
		if (err == NO_ERROR || err == ERROR_MORE_DATA)
		{
			for (DWORD i = 0; i < cEntries; i++)
			{
				if (pnr[i].lpLocalName != nullptr)
				{
					QString name = QString::fromWCharArray( pnr[i].lpRemoteName ).remove("\\\\TSCLIENT\\", Qt::CaseInsensitive);
					if ( ! names->contains( name ) )
					{
						drives->append( QString::fromWCharArray( pnr[i].lpLocalName ) );
						names->append( name );
						remove->append( false );
					}
				}
			}
			Settings::logInfo(QString("Appended existent drives; names: %1 drives: %2").arg(names->join(",")).arg(drives->join(",")));
			for (DWORD i = 0; i < cEntries; i++)
			{
				QString name = QString::fromWCharArray( pnr[i].lpRemoteName ).remove("\\\\TSCLIENT\\", Qt::CaseInsensitive);
				Settings::logInfo(QString("Checking name: %1").arg(name));
				if ( ! names->contains( name, Qt::CaseInsensitive ) )
				{
					QString drive = assignedDrives.value( name.toLower(), QString() );

					drive = appendRemoteDrive( name, drive );

					if ( ! drive.isEmpty() )
					{
						names->append( name );
						drives->append( drive );
						remove->append( true );
						Settings::logInfo(QString("Appending new drive: %1 for resource: %2").arg(drive).arg(name));
					} else {
						Settings::logInfo(QString("Not appending new drive for resource: %2").arg(name));
					}
				} else {
					Settings::logInfo(QString("Not appending drive as it has been appended previously. Name: %1").arg(name));
				}
			}
		} else {
			if ( err == ERROR_NO_MORE_ITEMS )
			{
				Settings::logInfo(QString("No items found enumerating net resource"));
			} else {
				Settings::logInfo(QString("Error enumerating net resource: %1").arg(err));
			}
		}
	} while (err == ERROR_MORE_DATA);

	LocalFree(buffer);
	WNetCloseEnum(hEnum);
#endif
}
