#ifndef SETTINGS_H
#define SETTINGS_H

#include <QSettings>
#include <QSet>
#include <QList>
#include <QString>
#include <QFile>
#include <QTcpSocket>
#include <QHostAddress>
#include <QDateTime>
#include <QNetworkInterface>
#include <QHostInfo>
#include <QMap>
#include <QWidget>

#include "constants.h"
#include "lwclient.h"

class Settings
{
	public:
        	static void init();

		static QHostAddress getLocalAddress();
		static void setLocalAddress(QHostAddress address);
		static int getLocalOs();
		static QString getOsName(int os);
		static bool isLocalhost(QString address);
		static int getLogLevel();

		static QMap<QString, QString> desktopSettings(QString file);

		static void setValue(QString file, QString key, QString value);
		static QString getValue(QString primaryFile, QString secondaryFile, QString key);
		static QString getValue(QString file, QString key);
		static QString getValue(QString key);
		static QStringList allKeys(QString file, QString group = QString());
		static bool getBooleanValue(QString key);
		static QStringList getIds(QString file);
		static QStringList getValues(QString file, QString field);
		static void setUserValue(QString key, QVariant value);
		static QVariant getUserValue(QString key, QVariant defaultValue);

		static QString getApplicationValue( QString appName, QString key );
		static QStringList getApplicationsList();

		static void saveWidgetGeometry(QString name, QWidget * widget);
		static void loadWidgetGeometry(QString name, QWidget * widget);

		static QString sharedPath();
		static QString searchPath();
		static QString windowsSharedPath();
		static QString windowsHomePath();
		static int ldapServerPort();

		static QString getUserName();
		static QString getPassword();
		static QString getServerAddress();
		static int getServerPort();
		static QString getProfilesServerAddress();
		static int getProfilesServerPort();
		static QString getFullUserName();
		static QString getDesktopServer();
		static QString getKioskApp();
		static QString getMimetype(QString extension);
		static QStringList getApplications( QString mimetype );
		static QStringList getFavoriteApplications();
		static void setFavoriteApplications(QStringList favoriteApps);
		static void setUserName(QString userName);
		static void setPassword(QString password);
		static void setServerAddress(QString address);
		static void setServerPort(int port);
		static void setExtensions();
		static int getWindowManagerBehaviour();
		static QString homePath();
		static void setHomePath( QString homePath );

		static void log(QString type, QString message);
		static void logInfo(QString message);
		static void logError(QString message);
		static void logDebug(QString message);
};

#endif
