#include "systraywidget.h"

SystrayWidget::SystrayWidget(QWidget *parent) : QWidget(parent)
{
	systrayManager = new SystrayManager( (HWND) this->winId(), this );
	connect( systrayManager, SIGNAL(addIcon(HWND)), this, SLOT(addIcon(HWND)));
	layout = new QHBoxLayout;
	layout->setDirection( QBoxLayout::RightToLeft );
	setLayout( layout );
	timer = new QTimer();
	timer->start( SYSTRAY_TIMER );
	connect( timer, SIGNAL(timeout()), this, SLOT(checkSystray()));
}

SystrayWidget::~SystrayWidget()
{
	timer->stop();
	delete timer;
	systrayManager->stop();
	delete systrayManager;
}

void SystrayWidget::childEvent(QChildEvent *event)
{
	/*switch ( event->type() )
	{
	case QChildEvent::ChildRemoved:
		qDebug("Widget Item removed!");
		break;
	case QChildEvent::ChildAdded:
		qDebug("Widget Item added!");
		break;
	default:
		qDebug("Widget Other event...");
		break;
	}*/
	QWidget::childEvent( event );
}

void SystrayWidget::addIcon(HWND winId)
{
	mutex.lock();
	if ( ! iconsWindowList.contains( winId ) )
	{
		iconsWindowList.append( winId );
	}
	mutex.unlock();
}

void SystrayWidget::removeIcon(HWND winId)
{
	SystrayClient * client = icons[winId];
	layout->removeWidget( client );
	delete client;
	icons.remove( winId );
	layout->update();
}

void SystrayWidget::removeLayout()
{
	mutex.lock();
	for ( int i = 0; i < iconsWindowList.count(); i++ )
	{
		removeIcon(iconsWindowList[i]);
	}
	delete layout;
	layout = new QHBoxLayout;
	setLayout( layout );
	mutex.unlock();
}

void SystrayWidget::updateIcons()
{
	QList<HWND> removeKeyList;

	mutex.lock();
	for ( int i = 0; i < iconsWindowList.count(); i++ )
	{
		if ( WinInfo::isValidWindow( iconsWindowList[i] ) )
		{
			if ( ! icons.contains( iconsWindowList[i] ) )
			{
				icons[iconsWindowList[i]] = new SystrayClient( iconsWindowList[i] );
				layout->insertWidget( 0, icons[iconsWindowList[i]], 1, Qt::AlignHCenter | Qt::AlignVCenter );
			}
		} else {
			removeKeyList += iconsWindowList[i];
		}
	}
	hide();
	qApp->processEvents();
	show();
	for ( int i = 0; i < removeKeyList.count(); i++ )
	{
		iconsWindowList.removeAll( removeKeyList[i] );
	}

	iconsWindowListImages.clear();
	for ( int i = 0; i < iconsWindowList.count(); i++ )
	{
		iconsWindowListImages.append( WinInfo::toPixmap( iconsWindowList[i] ).toImage() );
	}
	mutex.unlock();
}

void SystrayWidget::checkSystray()
{
	mutex.lock();
	bool check = false;

	if ( iconsWindowList.count() != iconsWindowListImages.count() )
	{
		check = true;
	} else {
		for ( int i = 0; i < iconsWindowList.count(); i++ )
		{
			if ( ! check && ! WinInfo::isValidWindow( iconsWindowList[i] ) )
			{
				check = true;
			}
			if ( ! check && iconsWindowListImages[i] != WinInfo::toPixmap( iconsWindowList[i] ).toImage() )
			{
				check = true;
			}
		}
	}
	mutex.unlock();

	if ( check )
	{
		removeLayout();
		updateIcons();
	}
}
