#ifndef TABBAR_H
#define TABBAR_H

#include <QPushButton>
#include <QHBoxLayout>
#include <QGridLayout>
#include <QString>
#include <QList>
#include <QIcon>
#include <QPixmap>
#include <QPainter>
#include <QFont>
#include <QWidget>
#include <QSize>
#include <QStackedWidget>
#include <QScrollArea>
#include <QScreen>
#include <QApplication>
#include <QGuiApplication>
#include <QTimer>
#include <QDate>
#include <QTime>
#include <QCalendarWidget>

#include "constants.h"
#include "threadbutton.h"
#include "threadwidget.h"
#include "desktop.h"
#include "configure.h"
#include "lwclient.h"
#include "winmanager.h"
#include "winaction.h"
#include "menu.h"
#include "systraywidget.h"
#include "initprogress.h"

class TabBar : public QWidget
{
	Q_OBJECT

	public:
		TabBar ( InitProgress * initProgress, QWidget * parent );
		~TabBar();

		void addButton(ThreadButton * button);

		ThreadButton * getButton( int index );
		QList<ThreadButton *> * getButtonList() { return buttonList; }
		Desktop * getDesktop() { return desktop; }
		Configure * getConfigure() { return configure; }
		QCalendarWidget * getCalendarWidget() { return calendarWidget; }

		int indexOf(HWND window, HWND railWindow = 0);
		int indexOf(QWidget * widget);
		int count();
		int threadButtonCount();
		int menuButtonCount();

	public slots:
		void updateApps();
		void removeButton( int index = -1 );
		void resize();
		void updateContents();

		void closeButtonClicked(ThreadButton * button);
		void killButtonClicked(ThreadButton * button);

		void setSelected( HWND window );
		void changeIcon( HWND window, QPixmap pixmap );

	private slots:
		void leftButtonClicked();
		void rightButtonClicked();
		void setDate();
		void dateClicked();
		void configureButtonClicked();
		void threadButtonClickedSlot(HWND window);
		void exitAppSlot();
		void newWindow(HWND window);
		void removeWindow(HWND window);
		void moveEvent(QMoveEvent *event);
		void resizeEvent(QResizeEvent *event);

	signals:
		void desktopButtonClicked();
		void buttonClicked(ThreadButton *);
		void threadButtonClicked(HWND);
		void closeButtonClicked(HWND);
		void killButtonClicked(HWND);
		void currentChanged( int );
		void launchApplication(Application*);
		void launchApplication();
		void newThread();
		void exitApp();

	private:
		void addMenuButtons();
		void showScrollButtons();
		void hideScrollButtons();
		ThreadButton * getFirstShownThreadButton();
		int numThreadButtons();
		void addToThreadsLayout( ThreadButton * button );
		void closeEvent(QCloseEvent *closeEvent);
		QList<HWND> getOwnWindows(bool includeOwnWidgets = true);

		QHBoxLayout * layout;
		QHBoxLayout * menusLayout;
		QHBoxLayout * threadsLayout;
		QHBoxLayout * exitLayout;

		ThreadWidget * currentThread;
		QWidget * threadsBar;
		QScrollArea * threadsScroll;
		QList<ThreadButton *> * buttonList;
		QPushButton * leftButton;
		QPushButton * rightButton;

		Menu * menu;
		Desktop * desktop;
		Configure * configure;

		InitProgress * initProgress;

		QPushButton * date;
		QCalendarWidget * calendarWidget;
		ThreadButton * exitButton;

		QWidget * lastMenuButton;
		int hiddenThreadButtons;
		int firstShownThreadButton;
};

#endif
