#include "tablewidget.h"

TableWidget::TableWidget(QWidget * parent) : QWidget( parent )
{
	columns = 0;
	rows = 0;

	setAttribute(Qt::WA_TranslucentBackground);

	createLayout();
}

void TableWidget::createLayout()
{
	layout = new QGridLayout;
	setLayout( layout );
	layout->setAlignment( Qt::AlignLeft | Qt::AlignTop );
	layout->setContentsMargins( 0, 0, 0, 0 );
	layout->setMargin( 0 );
	setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Maximum);
	//layout->setSizeConstraint(QLayout::SetNoConstraint);
}

void TableWidget::clear()
{
	delete layout;

	createLayout();
}

void TableWidget::addWidget(QWidget * widget, int row, int col)
{
	layout->addWidget(widget, row, col);
}

void TableWidget::addMultiCellWidget( QWidget * widget, int xPos, int yPos, int numRows, int numCols )
{
	int row = xPos;
	int column = yPos;

	if ( xPos < 0 )
	{
		row = rows + xPos - numRows + 1;
	}

	if ( yPos < 0 )
	{
		column = columns + yPos - numCols + 1;
	}

	qDebug("Widget size: %d x %d, row: %d, column: %d, numRows: %d, numColumns: %d", numCols * LISTVIEWITEM_GRID_SIZE.width() - 10, numRows * LISTVIEWITEM_GRID_SIZE.height() - 10, row, column, numRows, numCols );
	widget->setFixedSize( numCols * LISTVIEWITEM_GRID_SIZE.width() - 10, numRows * LISTVIEWITEM_GRID_SIZE.height() - 10);
	layout->addWidget( widget, row, column, numRows, numCols );
}

void TableWidget::removeWidget( QWidget * w )
{
	layout->removeWidget( w );
}

QWidget * TableWidget::widgetAtPosition(int row , int col)
{
	return layout->itemAtPosition(row, col)->widget();
}

void TableWidget::removeItemAtPosition(int row, int col)
{
	layout->removeItem(layout->itemAtPosition(row, col));
}

void TableWidget::addItem( QLayoutItem * item, int row, int col )
{
	layout->addItem( item, row, col );
}

void TableWidget::prepareCells(int numberOfApps)
{
	int cell_width = LISTVIEWITEM_GRID_SIZE.width();
	int cell_height = LISTVIEWITEM_GRID_SIZE.height();

	rows = height() / cell_height;
	columns = numberOfApps / rows;
	if ( rows * columns < numberOfApps )
	{
		columns++;
	}

	setFixedWidth( columns * cell_width );
	qDebug("Colums: %d, Fixed Width to: %d", columns, columns * cell_width );
	/*int w = width();
	int h = height();

	columns = (int) ((double)w / (double) cell_width);
	rows = (int) ((double)h / (double) cell_height);

	int vertical_space = ( w - (columns * cell_width) ) / columns;
	int horizontal_space = ( h - (rows * cell_height) ) / rows;
	layout->setVerticalSpacing( vertical_space );
	layout->setHorizontalSpacing( horizontal_space );

	qDebug("Desktop size: %d x %d, Rows: %d, Columns: %d, Vertical space: %d, Horizontal space: %d, Cell width: %d, Cell height: %d", w, h, rows, columns, vertical_space, horizontal_space, cell_width, cell_height);
*/
	for (int i=0; i<columns; i++)
	{
		for (int j=0; j<rows; j++)
		{
			QSpacerItem * spacer = new QSpacerItem(cell_width, cell_height, QSizePolicy::Preferred, QSizePolicy::Fixed);
			layout->addItem(spacer, j, i);
		}
	}
}

bool TableWidget::isEmpty(int row, int column)
{
	return layout->itemAtPosition(row, column)->isEmpty();
}

void TableWidget::mousePressEvent(QMouseEvent *event)
{
	emit mousePressed();
	QWidget::mousePressEvent( event );
}
