#include "winmanager.h"

#include "manager.h"

#ifdef Q_OS_LINUX
#include <QX11Info>
#include <X11/X.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/Xlib.h>
#endif

static QList<HWND> ownWindows;
static HWND penultimateTopMostWindow = (HWND) 0;
static HWND lastTopMostWindow = (HWND) 0;
static HWND lastMinimizedTopMostWindow = (HWND) 0;
//static HWND desktopWindow = (HWND) 0;

WinManager::WinManager( QObject * parent ) : QThread( parent )
{
	keepRunning = true;
	lastMinimizedTopMostWindow = (HWND) 0;
}

void WinManager::run()
{
	mutex.lock();
	bool _keepRunning = keepRunning;
	mutex.unlock();

	while ( _keepRunning )
	{
		QList<HWND> windows = WinInfo::getApplicationWindows();

		manageApplicationWindows( windows );

		QThread::msleep( WINMANAGER_TIMER );

		mutex.lock();
		_keepRunning = keepRunning;
		mutex.unlock();
	}
}

void WinManager::stop()
{
	mutex.lock();
	keepRunning = false;
	mutex.unlock();
}

void WinManager::setDesktopWindow( HWND window )
{
	appendOwnWindow( window );
//	desktopWindow = window;
}

void WinManager::appendOwnWindow( HWND window )
{
	ownWindows += window;
}

QList<HWND> WinManager::getOwnWindows()
{
	return ownWindows;
}

HWND WinManager::getPenultimateTopMostWindow()
{
	return penultimateTopMostWindow;
}

HWND WinManager::getLastTopMostWindow()
{
	return lastTopMostWindow;
}

HWND WinManager::getLastMinimizedTopMostWindow()
{
	return lastMinimizedTopMostWindow;
}

void WinManager::manageApplicationWindows( QList<HWND> windows )
{
	for ( int i = 0; i < runningApplications.count(); i++ )
	{
		if ( ! windows.contains( runningApplications[i] ) )
		{
			windowIcon.remove( runningApplications[i] );
			emit removeApplication( runningApplications[i] );
		}
	}

	for ( int i = 0; i < windows.count(); i++ )
	{
		if ( ! runningApplications.contains( windows[i] ) )
		{
			emit newApplication( windows[i] );
		}
	}

	runningApplications = windows;

	checkTopMostWindow();

	checkIcons();
}

void WinManager::checkTopMostWindow()
{
	HWND _lastTopMostWindow = WinInfo::getTopMostWindow();
	if ( ownWindows.contains( _lastTopMostWindow) )
	{
		if ( lastTopMostWindow != 0 )
		{
			penultimateTopMostWindow = lastTopMostWindow;
			lastMinimizedTopMostWindow = lastTopMostWindow;
		}
		lastTopMostWindow = 0;
	} else {
		if ( WinInfo::isApplication( _lastTopMostWindow ) )
		{
			penultimateTopMostWindow = _lastTopMostWindow;
			lastMinimizedTopMostWindow = lastTopMostWindow;
			if ( ! WinInfo::isMinimized( _lastTopMostWindow ) )
			{
				lastTopMostWindow = _lastTopMostWindow;
			} else {
				lastTopMostWindow = 0;
			}
		} else {
			penultimateTopMostWindow = _lastTopMostWindow;
			lastTopMostWindow = WinInfo::getParent( _lastTopMostWindow );
		}
	}

	emit setSelected( lastTopMostWindow );
}

void WinManager::checkIcons()
{
	for ( int i = 0; i < runningApplications.count(); i++ )
	{
		QPixmap icon = WinInfo::getIcon( runningApplications[i] );
		QImage iconImage = icon.toImage();
		if ( windowIcon[ runningApplications[i] ] != iconImage )
		{
			windowIcon[ runningApplications[i] ] = iconImage;
			emit iconChanged( runningApplications[i], icon );
		}
	}
}

void WinManager::desktopRequested()
{
	QList<HWND> _lastMinimized;
	QList<HWND> ownWindows = getOwnWindows();

	for ( int i = 0; i < runningApplications.count(); i++ )
	{
		if ( ! WinInfo::isMinimized( runningApplications[i] ) && ! ownWindows.contains( runningApplications[i] ) )
		{
			WinAction::minimizeWindow( runningApplications[i] );
			_lastMinimized += runningApplications[i];
		}
	}

	if ( _lastMinimized.count() == 0 )
	{
		for ( int i = 0; i < lastMinimized.count(); i++ )
		{
			if ( WinInfo::isValidWindow( lastMinimized[i] ) && lastMinimized[i] != lastMinimizedTopMostWindow )
			{
				WinAction::showWindow( lastMinimized[i] );
			}
		}
                lastMinimized.clear();

		if ( WinInfo::isValidWindow( lastMinimizedTopMostWindow ) )
		{
			WinAction::showWindow( lastMinimizedTopMostWindow );
		}
	} else {
		lastMinimized = _lastMinimized;
	}
}
