#ifndef WINMANAGER_H
#define WINMANAGER_H

#include <QThread>
#include <QObject>
#include <QMutex>
#include <QColor>
#include <QDebug>
#include <QWindow>
#include <QPixmap>
#include <QImage>

#ifdef Q_OS_WIN
#include <WinSock2.h>
#include <Windows.h>
#include <Psapi.h>
#include <LM.h>
#include <WtsApi32.h>
#include <QtWinExtras/QtWin>
#endif

#include "wininfo.h"
#include "constants.h"

Q_DECLARE_METATYPE(QList<HWND>)

class WinManager : public QThread
{
	Q_OBJECT

	public:
		WinManager( QObject * parent = Q_NULLPTR );

		void run();
		void stop();

		static void appendOwnWindow( HWND window );
		static void setDesktopWindow( HWND window );
		static QList<HWND> getOwnWindows();
		static HWND getPenultimateTopMostWindow();
		static HWND getLastTopMostWindow();
		static HWND getLastMinimizedTopMostWindow();


	public slots:
		void desktopRequested();

	signals:
		void newApplication( HWND window );
		void removeApplication( HWND window );
		void iconChanged( HWND window, QPixmap icon );
		void setSelected( HWND window );

	private:
		void manageApplicationWindows( QList<HWND> windows );
		void checkTopMostWindow();
		void checkIcons();

		bool keepRunning;
		QMutex mutex;
		QList<HWND> lastMinimized;
		QList<HWND> runningApplications;
		QMap<HWND,QImage> windowIcon;
};

#endif
