#include <QFontDatabase>
#include <QFont>
#include <QDir>
#include <QStringList>
#include <QString>
#include <QList>
#include <QFile>
#include <QApplication>

#include <iostream>

#include "../common/constants.h"
#include "../common/configure.h"

#ifdef Q_OS_WIN
#include <windows.h>
#include <fcntl.h>
#include <io.h>
#else
#include <termios.h>
#include <unistd.h>
#endif

void setStdinEcho(bool enable = true)
{
#ifdef Q_OS_WIN
	HANDLE hStdin = GetStdHandle(STD_INPUT_HANDLE); 
	DWORD mode;
	GetConsoleMode(hStdin, &mode);

	if( ! enable )
	{
		mode &= ~ENABLE_ECHO_INPUT;
	} else {
		mode |= ENABLE_ECHO_INPUT;
	}

	SetConsoleMode(hStdin, mode );
#else
	struct termios tty;
	tcgetattr(STDIN_FILENO, &tty);
	if( !enable )
	{
		tty.c_lflag &= ~ECHO;
	} else {
		tty.c_lflag |= ECHO;
	}
	(void) tcsetattr(STDIN_FILENO, TCSANOW, &tty);
#endif
}

void usage(QString commandName, QString error = QString())
{
	if ( ! error.isEmpty() )
	{
		qDebug("Error: %s", error.toUtf8().data());
	}
	qDebug("Usage: %s", commandName.toUtf8().data());
	qDebug("Example: %s", commandName.toUtf8().data());
	exit(1);
}

int main(int argc, char ** argv)
{
	QApplication qApplication(argc, argv);

	QDir fontsDir( LIBERUS_FONT_DIR );
	QStringList fontFile = fontsDir.entryList(QStringList("*.ttf"));
	QList<int> fontId;
	for (int i = 0; i < fontFile.count(); i++)
	{
		fontId.append( QFontDatabase::addApplicationFont( LIBERUS_FONT_DIR + QString(QDir::separator()) + fontFile.at(i) ) );
	}
	QFontDatabase fontDatabase;
	QString family = QFontDatabase::applicationFontFamilies(fontId.at(0)).at(0);
	QFont font = fontDatabase.font( Settings::getUserValue("Font", LIBERUS_FONT).toString(), QString(), Settings::getUserValue("FontSize", QString::number(LIBERUS_FONT_BASE_SIZE)).toInt());
	qApplication.setFont( font );

#ifdef Q_OS_WIN32
		_setmode(_fileno(stdin), _O_BINARY);
#endif
	Configure * configure = new Configure();
	configure->addApplications();

	configure->show();

	return qApplication.exec();
}
