#include "testmanager.h"

#ifdef Q_OS_LINUX
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#endif

TestManager::TestManager( QListWidget * widget, QObject * parent ) : QThread( parent )
{
	keepRunning = true;
	this->widget = widget;
}

void TestManager::run()
{
	mutex.lock();
	bool _keepRunning = keepRunning;
	mutex.unlock();

	while ( _keepRunning )
	{
		QList<HWND> windows = WinInfo::getApplicationWindows();
		HWND activeWindow = WinInfo::getTopMostWindow();
		HWND parent = WinInfo::getParent( activeWindow );
		qDebug("Active window: %d, parent: %d", (int)activeWindow, (int)parent);

		/*QPixmap pixmap = WinInfo::toPixmap( activeWindow );
		QLabel * image = new QLabel( 0 );
		image->setFixedSize( pixmap.size() );
		image->setPixmap( pixmap );
		image->show();*/

		for ( int i = 0; i < windows.count(); i++ )
		{
			QString title = WinInfo::getTitle( windows[i] );
			if ( title == "Liberus Desktop Bar" )
			{
				title = "Dock: " + title + " WinId: " + QString::number( windows[i] );
				setDockBar( windows[i] );
			}
			if ( title == "Liberus Desktop" )
			{
				title = "Desktop: " + title + " WinId: " + QString::number( windows[i] );
				setDesktop( windows[i] );
			}
			if ( windows[i] == activeWindow )
			{
				title = "* " + title;
			}
			if ( WinInfo::isMinimized( (HWND) windows[i] ) )
			{
				title = "(minimized) " + title;
			}
			new QListWidgetItem( QIcon(WinInfo::getIcon( windows[i] )), title, widget );
		}

		QThread::msleep( 5000 );

		widget->clear();

		mutex.lock();
		_keepRunning = keepRunning;
		mutex.unlock();
	}
}

void TestManager::stop()
{
	mutex.lock();
	keepRunning = false;
	mutex.unlock();
}

void TestManager::setDesktop( HWND id )
{
	Display * display  = QX11Info::display();
        Atom tmp = XInternAtom(display, "_NET_WM_WINDOW_TYPE_DESKTOP", False);
        XChangeProperty(display,
                id,
                XInternAtom(display, "_NET_WM_WINDOW_TYPE", False),
                XA_ATOM ,
                32,
                PropModeReplace,
                (unsigned char *)&tmp, 1);
	Atom desktopGeometry = XInternAtom(display, "_NET_DESKTOP_GEOMETRY", False);
        QSize geometry = QSize( 1620, 1080 - TAB_BUTTON_HEIGHT );

        uint32_t data[5] = { uint32_t(geometry.width()), uint32_t(geometry.height()), 0, 0, 0};

        WinAction::sendClientMessage( SubstructureRedirectMask|SubstructureNotifyMask, XDefaultRootWindow(display), id, desktopGeometry, data);

	
	XChangeProperty(display,
		id,
		desktopGeometry,
		XA_ATOM,
		32,
		PropModeReplace,
		(unsigned char *)data, 1);
}

void TestManager::setDockBar( HWND id )
{
	QSize desktopSize = QSize( 1620, 1080 - TAB_BUTTON_HEIGHT );
	Display * display  = QX11Info::display();

        // Define type of window as dock
        Atom tmp = XInternAtom(display, "_NET_WM_WINDOW_TYPE_DOCK", False);
        XChangeProperty(display,
                id,
                XInternAtom(display, "_NET_WM_WINDOW_TYPE", False),
                XA_ATOM,
                32,
                PropModeReplace,
                (unsigned char *)&tmp, 1);

        // Show on all virtual desktops
        long data[1] = { 0xFFFFFFFF };
        XChangeProperty(display,
                id,
                XInternAtom(display, "_NET_WM_DESKTOP", False),
                XA_CARDINAL,
                32,
                PropModeReplace,
                (unsigned char *) &data, 1);

        // left, right, top, bottom,
        // left_start_y, left_end_y, right_start_y, right_end_y, top_start_x, top_end_x, bottom_start_x, bottom_end_x
        // [0, 0, bar_size, 0, 0, 0, 0, 0, x, x+width-1, 0, 0]
        // QApplication::primaryScreen()->size().height() - TAB_BUTTON_HEIGHT

        int insets[12] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
        insets[2] = desktopSize.height() + 1;
	insets[6] = desktopSize.height() + 1;
	insets[7] = desktopSize.height() + 1;
        insets[8] = 0;
        insets[9] = desktopSize.width();

        XChangeProperty(display,
                XDefaultRootWindow(QX11Info::display()),
                XInternAtom(display, "_NET_WM_STRUT", False),
                XA_CARDINAL ,
                32,
                PropModeReplace,
                (unsigned char *)&insets, 4);

        XChangeProperty(display,
                XDefaultRootWindow(QX11Info::display()),
                XInternAtom(display, "_NET_WM_STRUT_PARTIAL", False),
                XA_CARDINAL ,
                32,
                PropModeReplace,
                (unsigned char *)&insets, 12);

        // Define work area
        int workArea[4] = { 0, 0, desktopSize.width(), desktopSize.height() };
        XChangeProperty(display,
                XDefaultRootWindow(QX11Info::display()),
                XInternAtom(display, "_NET_WORKAREA", False),
                XA_CARDINAL ,
                32,
                PropModeReplace,
                (unsigned char *)&workArea, 4);

        // Move dock to its place
        WinAction::locateWindow( id, 0, desktopSize.height(), desktopSize.width(), TAB_BUTTON_HEIGHT );
}
